﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.ApplicationServices;
using System.Collections;
using Autodesk.AutoCAD.GraphicsInterface;

[assembly:ExtensionApplication(typeof(OTIS_Testing.Commands))]
namespace OTIS_Testing
{
    public class Commands : IExtensionApplication
    {
        public Commands()
        {
            publicConfig = new PublicConifg();
        }

        public void Initialize()
        {

        }

        public void Terminate()
        {

        }

        static PublicConifg publicConfig;

        string RightBlockName { get { return "Hanna_Right"; } }

        string FrontBlockName { get { return "Hanna_Front"; } }

        string BottomBlockName { get { return "Hanna_Bottom"; } }

        string FrameBlockName { get { return "GB-A3"; } }

        string ListBlockName { get { return "ParameterTable"; } }

        string TitleBlockName { get { return "ZWX"; } }

        /// <summary>
        /// insert block reference
        /// </summary>
        /// <param name="db"></param>
        /// <param name="insertPoint"></param>
        /// <param name="blockName"></param>
        private void InsertBlock(Database db, Transaction tr, Point3d insertPoint, string blockName, bool isUpdateProeprty)
        {
            ObjectId blockRefeId = ObjectId.Null;
            BlockTable bt = tr.GetObject(db.BlockTableId, OpenMode.ForRead) as BlockTable;
            BlockTableRecord btr = tr.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite) as BlockTableRecord;
            BlockReference blockRefe = new BlockReference(insertPoint, bt[blockName]);
            blockRefeId = btr.AppendEntity(blockRefe);
            tr.AddNewlyCreatedDBObject(blockRefe, true);
            if (blockName == ListBlockName)
                AddAttrubiteReference(db, tr, blockRefe);
            if (isUpdateProeprty)
                UpdateDynamicBlockProperty(db, tr, blockRefe);
            //UpdaetBlockDefinition(tr, bt[blockName]);
            //btr.UpdateAnonymousBlocks();
        }

        /// <summary>
        /// 更新块属性
        /// </summary>
        /// <param name="blockId"></param>
        /// <param name="config"></param>
        private void UpdateDynamicBlockProperty(Database db, Transaction tr, BlockReference blockR)
        {
            #region update dynamic Parameters
            DynamicBlockReferencePropertyCollection blockPropertys = blockR.DynamicBlockReferencePropertyCollection;
            foreach (DynamicBlockReferenceProperty property in blockPropertys)
            {
                if (property.ReadOnly)
                    continue;
                switch (property.PropertyName)
                {
                    case "Hole_Smaildia":
                        property.Value = publicConfig.Hole_Smaildia;
                        break;
                    case "Hole_Bigdia":
                        property.Value = publicConfig.Hole_Bigdia;
                        break;
                    case "Block_Thick":
                        property.Value = publicConfig.Block_Thick;
                        break;
                    case "Plate_Holeradius":
                        property.Value = publicConfig.Plate_Holeradius;
                        break;
                    case "Plate_Length":
                        property.Value = publicConfig.Plate_Length;
                        break;
                    case "Plate_Thick":
                        property.Value = publicConfig.Plate_Thick;
                        break;
                    case "Plate_Width":
                        property.Value = publicConfig.Plate_Width;
                        break;
                    case "Plate_Height":
                        property.Value = publicConfig.Plate_Height;
                        break;
                }
            }
            #endregion

            #region update Attribuate
            AttributeCollection attrubate = blockR.AttributeCollection;
            foreach (ObjectId attid in attrubate)
            {
                AttributeReference ar = tr.GetObject(attid, OpenMode.ForWrite) as AttributeReference;
                switch (ar.Tag)
                {
                    case "Hole_Smalldia":
                        ar.TextString = publicConfig.Hole_Smaildia.ToString();
                        break;
                    case "Hole_Bigdia":
                        ar.TextString = publicConfig.Hole_Bigdia.ToString();
                        break;
                    case "BLOCK_THICK":
                        ar.TextString = publicConfig.Block_Thick.ToString();
                        break;
                    case "Plate_Holeradius":
                        ar.TextString = publicConfig.Plate_Holeradius.ToString();
                        break;
                    case "Plate_Length":
                        ar.TextString = publicConfig.Plate_Length.ToString();
                        break;
                    case "Plate_Thick":
                        ar.TextString = publicConfig.Plate_Thick.ToString();
                        break;
                    case "Plate_Width":
                        ar.TextString = publicConfig.Plate_Width.ToString();
                        break;
                    case "Plate_Height":
                        ar.TextString = publicConfig.Plate_Height.ToString();
                        break;
                }
            }
            #endregion

        }

        /// <summary>
        /// 更新属性信息
        /// </summary>
        /// <param name="blockR"></param>
        private void UpdateAttrubite(Transaction tr,BlockReference blockR)
        {
            BlockTableRecord btr = tr.GetObject(blockR.BlockTableRecord, OpenMode.ForRead) as BlockTableRecord;
            foreach (ObjectId attid in btr)
            {
                Entity entity = tr.GetObject(attid, OpenMode.ForRead) as Entity;
                if (entity is AttributeDefinition)
                {
                    AttributeDefinition attribute = entity as AttributeDefinition;
                    AttributeReference ar = new AttributeReference();
                    switch (attribute.Tag)
                    {
                        case "Hole_Smalldia":
                            ar.TextString = publicConfig.Hole_Smaildia.ToString();
                            break;
                        case "Hole_Bigdia":
                            ar.TextString = publicConfig.Hole_Bigdia.ToString();
                            break;
                        case "Block_Thick":
                            ar.TextString = publicConfig.Block_Thick.ToString();
                            break;
                        case "Plate_Holeradius":
                            ar.TextString = publicConfig.Plate_Holeradius.ToString();
                            break;
                        case "Plate_Length":
                            ar.TextString = publicConfig.Plate_Length.ToString();
                            break;
                        case "Plate_Thick":
                            ar.TextString = publicConfig.Plate_Thick.ToString();
                            break;
                        case "Plate_Width":
                            ar.TextString = publicConfig.Plate_Width.ToString();
                            break;
                        case "Plate_Height":
                            ar.TextString = publicConfig.Plate_Height.ToString();
                            break;
                    }
                    ar.SetAttributeFromBlock(attribute, blockR.BlockTransform);
                    ar.Position = attribute.Position.TransformBy(blockR.BlockTransform);
                    blockR.AttributeCollection.AppendAttribute(ar);
                }
            }
        }

        /// <summary>
        /// add attrubite
        /// </summary>
        /// <param name="db"></param>
        /// <param name="tr"></param>
        /// <param name="blockR"></param>
        private void AddAttrubiteReference(Database db, Transaction tr, BlockReference blockR)
        {
            BlockTableRecord btr = tr.GetObject(blockR.BlockTableRecord, OpenMode.ForRead) as BlockTableRecord;
            foreach (ObjectId attid in btr)
            {
                Entity entity = tr.GetObject(attid, OpenMode.ForRead) as Entity;
                if (entity is AttributeDefinition)
                {
                    AttributeDefinition attribute = entity as AttributeDefinition;
                    AttributeReference ar = new AttributeReference();
                    ar.SetAttributeFromBlock(attribute, blockR.BlockTransform);
                    ar.Position = attribute.Position.TransformBy(blockR.BlockTransform);
                    blockR.AttributeCollection.AppendAttribute(ar);
                }
            }
        }

        private void UpdaetBlockDefinition(Transaction tr, ObjectId blockId)
        {
            BlockTableRecord btrentity = tr.GetObject(blockId, OpenMode.ForRead) as BlockTableRecord;
            foreach (ObjectId attid in btrentity)
            {
                Entity entity = tr.GetObject(attid, OpenMode.ForRead) as Entity;
                entity.UpgradeOpen();
                entity.DowngradeOpen();
                //if (entity is Hatch)
                //{
                //    Hatch hatchEntity = entity as Hatch;
                //    hatchEntity.UpgradeOpen();
                //    hatchEntity.DowngradeOpen();
                //}
            }
        }

        [CommandMethod("GenDraw")]
        public void GeneratedDrawing()
        {
            try
            {
                ConfigForm configForm = new ConfigForm(publicConfig);
                Autodesk.AutoCAD.ApplicationServices.Application.ShowModalDialog(configForm);
                if (!configForm.isOK)
                    return;
                Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
                if (doc == null)
                    return;
                Database db = HostApplicationServices.WorkingDatabase;
                Point3d insertPoint;
                using (db)
                {
                    using (Transaction tr = db.TransactionManager.StartTransaction())
                    {
                        insertPoint = new Point3d(0, 0, 0);
                        InsertBlock(db,tr, insertPoint, FrameBlockName, false);
                        insertPoint = new Point3d(415, 5, 0);
                        InsertBlock(db,tr, insertPoint, TitleBlockName, false);
                        insertPoint = new Point3d(415, 50, 0);
                        InsertBlock(db,tr, insertPoint, ListBlockName, true);
                        insertPoint = new Point3d(124.6, 134.5, 0);
                        InsertBlock(db,tr, insertPoint, RightBlockName, true);
                        insertPoint = new Point3d(243, 134.5, 0);
                        InsertBlock(db,tr, insertPoint, FrontBlockName, true);
                        insertPoint = new Point3d(124.6, 106, 0);
                        InsertBlock(db, tr, insertPoint, BottomBlockName, true);
                        tr.Commit();
                        //db.TransactionManager.QueueForGraphicsFlush();
                    }
                }
                Application.DocumentManager.MdiActiveDocument.SendStringToExecute("DIMREGEN ", true, false, false);
                Application.DocumentManager.MdiActiveDocument.SendStringToExecute("REGENALL ", true, false, false);
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex)
            {

            }
            catch (System.Exception e)
            {

            }
        }

        [CommandMethod("UpdateDraw")]
        public void UpdateDrawing()
        {
            try
            {
                ConfigForm configForm = new ConfigForm(publicConfig);
                Autodesk.AutoCAD.ApplicationServices.Application.ShowModalDialog(configForm);
                if (!configForm.isOK)
                    return;
                Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
                if (doc == null)
                    return;
                Database db = HostApplicationServices.WorkingDatabase;
                using (db)
                {
                    using (Transaction tr = db.TransactionManager.StartTransaction())
                    {
                        BlockTable bt = tr.GetObject(db.BlockTableId, OpenMode.ForRead) as BlockTable;
                        BlockTableRecord btr = tr.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForRead) as BlockTableRecord;
                        foreach (ObjectId oid in btr)
                        {
                            Entity entity = tr.GetObject(oid, OpenMode.ForRead) as Entity;
                            if (entity is BlockReference)
                            {
                                BlockReference blr = entity as BlockReference;
                                blr.UpgradeOpen();
                                UpdateDynamicBlockProperty(db, tr, blr);
                                blr.DowngradeOpen();
                            }
                        }
                        tr.Commit();
                        //db.TransactionManager.QueueForGraphicsFlush();
                    }
                }
                Application.DocumentManager.MdiActiveDocument.SendStringToExecute("DIMREGEN ", true, false, false);
                Application.DocumentManager.MdiActiveDocument.SendStringToExecute("REGENALL ", true, false, false);
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex)
            {

            }
            catch (System.Exception e)
            {

            }

        }
    }

    public class PublicConifg
    {

        public PublicConifg()
        {
            Hole_Smaildia = 12;
            Hole_Bigdia = 20;
            Block_Thick = 10;
            Plate_Holeradius = 8;
            Plate_Length = 60;
            Plate_Thick = 30;
            Plate_Width = 80;
            Plate_Height = 120;
        }

        /// <summary>
        /// 小孔径
        /// </summary>
        public double Hole_Smaildia { get; set; }

        /// <summary>
        /// 大孔径
        /// </summary>
        public double Hole_Bigdia { get; set; }
         
        /// <summary>
        /// 侧板厚度
        /// </summary>
        public double Block_Thick { get; set; }

        /// <summary>
        /// 孔半径
        /// </summary>
        public double Plate_Holeradius { get; set; }

        /// <summary>
        /// 侧板长度
        /// </summary>
        public double Plate_Length { get; set; }
        
        /// <summary>
        /// 底板厚度
        /// </summary>
        public double Plate_Thick { get; set; }
        
        /// <summary>
        /// 板宽度
        /// </summary>
        public double Plate_Width { get; set; }

        /// <summary>
        /// 板高度
        /// </summary>
        public double Plate_Height { get; set; }

        /// <summary>
        /// 验证
        /// </summary>
        /// <returns></returns>
        public bool Valida()
        {
            return true;
        }
    }


}
